#!/usr/bin/env python3
import os
import magic
import pathlib


class NoFileOrDirectory(Exception):
    """
    Exception raised when user did not give a complete path
    to the current zathura file
    """

    def __init__(
        self,
        dir_path,
        file_path,
        message="ERROR: No directory or file found",
    ):
        self.file = file_path
        self.dir = dir_path
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.message}\nFile: "{self.file}"\nDir: "{self.dir}"'


def filter_file(test_file):
    """
    Returns true if file is not a known file type zathura
    """
    mime = magic.Magic(mime=True)
    file_type = mime.from_file(test_file)
    accepted_types = [
        "image/vnd.djvu",
        "application/epub+zip",
        "application/postscript",
        "application/zip",
        "application/pdf",
    ]
    if file_type in accepted_types:
        return False

    return True


def get_next_or_prev_file(current_file_path, next_file=False, prev_file=False):
    """
    Get the next file in a directory when given a file path
    """
    # Split the path into directory and file
    temp = os.path.split(current_file_path)
    directory = temp[0]
    current_file = temp[1]
    # Check that both directory and file are not empty
    if not directory or not current_file:
        # If they are empty raise an error and exit with status 1
        raise NoFileOrDirectory(dir_path=directory, file_path=current_file)

    # Get a (sorted) list of files from the given directory
    file_list = os.listdir(directory)
    file_list.sort()

    # Populate vars
    new_index = 0
    incrementor = 0
    range_limit = 0
    reset_val = 0

    # Set vars depending on next or prev
    if next_file:
        incrementor = 1
        range_limit = len(file_list)
        reset_val = 0
    elif prev_file:
        incrementor = -1
        range_limit = -1
        reset_val = len(file_list) - 1

    # Add one to the given files index to get the next file
    if current_file in file_list:
        new_index = file_list.index(current_file) + incrementor

    # Check if we are out of bounds
    if next_file and new_index >= len(file_list):
        new_index = reset_val
    elif prev_file and new_index < 0:
        new_index = reset_val

    next_or_prev_file = os.path.join(directory, file_list[new_index])

    # Test if next_or_prev_file is valid file
    for i in range(new_index, range_limit, incrementor):
        next_or_prev_file = os.path.join(directory, file_list[i])
        if os.path.isfile(next_or_prev_file) and not filter_file(next_or_prev_file):
            return next_or_prev_file

    # File was not found so we start from the beginning / end
    new_index = reset_val
    for i in range(new_index, range_limit, incrementor):
        next_or_prev_file = os.path.join(directory, file_list[i])
        if os.path.isfile(next_or_prev_file) and not filter_file(next_or_prev_file):
            return next_or_prev_file

    # If the current file is the only compatible file then return None
    return None


def write_pid_file(pid_file, current_file):
    with open(pid_file, "w") as pf:
        pf.write(current_file)
    return 0



#!/usr/bin/env python3
import csv
import os
import pathlib
import re
import sys
from argparse import ArgumentParser
from shlex import split
from typing import Optional, Iterable


class UserSettings:
    """
    Easily enable users to personalize your program to their liking.
    Arguments:
        program[str]:
            The name of your program. This is used to define where user
            config files will be stored, e.g. $XDG_CONIG_HOME/program/.

        cmd_line_args(Optional[ArgumentParser]):
            Takes an ArgumentParser object and stores those values under
            user.cmd_args. If you use argparse it makes it easy to store
            that object within this one. One less thing to juggle.

        user_files(Optional[dict]):
            This should be a dict of the user files you want created (if
            needed). So if the user_files dict looks like this:
                {"conf_file": "my_program.conf"}
            Then my_program.conf will be created in the defined config dir
            for your program (see defined variables). You will also be able
            to access this value latter:
                >>> user = UserSettings(my_program, user_files=user_files)
                >>> user.conf_file
                '/home/user/.config/program/my_program.conf'

        create_files(Optional[bool]):
            If you don't want UserSettings to create the files in user_files
            then set this to false. The default value is True.

        default_settings(Option[dict]):
            This should be a dict with settings that a user may define or that
            you may find useful along with their default settings if a user
            were not to define them.
                >>> settings = {"favorite_color": "blue"}
                >>> user = UserSettings("my_program", default_settings=settings)
                >>> user.favorite_color
                'blue'

        read_files(Op[list]):
            If user_files was given then read_files can be a list of the keys
            whose file values you want parsed for user settings. So if my
            user_files looks like this: {"conf_file": "my_program.conf"}
            then I can pass read_files with this value: ["conf_file"]
            UserSettings will then parse that file. Speaking of parsing, let's
            talk about that.

    Parsing files:
        UserSettings can parse any user_file to update the UserSettings objects
        user_settings values. This is done by a simple csv parser. As users
        usually work with conf files and not csv files (in the context of user
        configurable cli programs) the default deliminator is the equal sign,
        "=". This is because it is clean and commonly found in conf files. It is
        probably simplest if I give an example. Take this file for instance:

            # my_program.conf
            # setting \= value
            fav_color = blue
            fav_bool = true
            fav_int = 5
            fav_sentence = my sentence without an equal sign
            other_sentence = my sentence with an escapped \= sign

        If a value in the first column was defined as a key in default_settings
        the value will be updated after parsing the desired file. Additionally
        UserSettings will make an attempt to detect any int or bool values. In
        the above example both fav_int and fav_bool will actually be an int and
        bool value respectively. Any value that does not evaluate to an int or
        bool will be treated as a string. UserSettings allows for escaped
        delimiters. If two unescapped delimiters are found in a line an error
        will be raised with the value of the line where the error was
        encountered. The white space/newlines are stripped from both columns.
        If a setting is not listed in default_settings then UserSettings ignores
        the setting and it's value.
    """

    def __init__(
        self,
        program: str,
        delimiter: Optional[str] = "=",
        cmd_line_args: Optional[ArgumentParser] = None,
        user_files: Optional[dict] = {},
        create_files: Optional[bool] = True,
        default_settings: Optional[dict] = {},
        read_files: Optional[list] = [],
    ):
        self.cmd_args = cmd_line_args
        self._user_files = user_files
        self._define_files(program=program, user_files=user_files)
        self._assert_files(create_files=create_files)
        self._set_default_settings(default_settings=default_settings)
        self._read_user_settings(
            user_settings=default_settings.keys(),
            read_files=read_files,
            delimiter=delimiter,
        )

    def _assert_files(self, create_files):
        """
        Test if config files exist and create them if needed
        """
        dir = pathlib.Path(self._conf_dir)
        if not dir.is_dir():
            dir.mkdir(parents=True, exist_ok=True)
        if not create_files:
            return
        for file in self._files:
            f = pathlib.Path(file)
            if not f.is_file():
                f.touch()

    def _define_files(self, program, user_files):
        """
        Set user config directory location and where user files should be
        located
        """
        path = os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~/.config"))
        self._conf_dir = os.path.join(path, program)
        self._files = []
        for key, value in user_files.items():
            file = os.path.join(self._conf_dir, value)
            self._files.append(file)
            self._user_files[key] = file
            setattr(self, key, file)

    def _read_user_settings(
        self,
        user_settings,
        read_files=None,
        delimiter="=",
    ):
        """
        Read the desired files and set setting values from parsed file.
        """
        # read_files should be a list of the key value(s) from the user_files
        # dict to read
        files = []
        for key, file in self._user_files.items():
            if key in read_files:
                files.append(file)
        # Regex object for
        r = re.compile(rf"\\{delimiter}")
        # Read the desired files
        for file in files:
            test_file = pathlib.Path(file)
            if not test_file.is_file():
                continue
            with open(file) as f:
                reader = csv.reader(
                    f,
                    delimiter=delimiter,
                    escapechar="\\",
                    quoting=csv.QUOTE_NONE,
                )
                for row in reader:
                    if len(row) > 2:
                        raise csv.Error(f"Too many fields on row: {row}")
                    elif len(row) < 2:
                        continue
                    setting_name = row[0].strip()
                    setting_name = r.sub("=", setting_name)
                    setting_value = row[1].strip()
                    setting_value = r.sub("=", setting_value)
                    try:
                        setting_value = eval(setting_value.capitalize())
                    except (NameError, SyntaxError):
                        pass
                    if not setting_name in user_settings:
                        continue
                    setattr(self, setting_name, setting_value)

    def _set_default_settings(self, default_settings):
        """
        Set default values for settings that user might not provide
        """
        for key, value in default_settings.items():
            setattr(self, key, value)


if __name__ == "__main__":
    __program__ = "znp"
    user_files = {
        "conf_file": f"{__program__}.conf",
        "log": f"{__program__}.log",
    }
    user_settings = ["prompt_cmd", "args"]
    read_files = ["conf_file"]
    default_settings = {
        "prompt_cmd": "fzf",
        "args": "",
    }
    user = UserSettings(
        program=__program__,
        user_files=user_files,
        default_settings=default_settings,
        read_files=read_files,
    )



#!/usr/bin/env python3
import re
from .promptx import PromptX, PromptXCmdError, PromptXError, PromptXSelectError


class InvalidCmdPrompt(Exception):
    """Exception raised when user has invalid command prompt"""

    def __init__(self, error, message="ERROR: prompt_cmd not recognized"):
        self.error = error
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.message} "{self.error}"'


class InputError(Exception):
    """Exception fzf or dmenu prompt fails"""

    def __init__(self, error, message="ERROR: Could not get user input", prefix=""):
        self.error = error
        self.prefix = prefix
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"{self.prefix}{self.message}{self.error}"


def zathura_prompt(pid_dict, prompt_cmd, prompt_args=None):
    """
    Prompt user to select their desired instance
    """
    # Create a list of options made up of the second value of
    # z.pid_dict. This looks like "123456: /path/to/file.pdf"
    options = []
    for val in pid_dict.values():
        options.append(val[1])
    # See if user wants to use custom prompt
    r = re.compile(r"(--prompt|-p\s)")
    prompt = None if r.findall(prompt_args) else "Use: "
    # Ask user about what instance to use
    if not prompt_args:
        prompt_args = "-l 20 -i" if prompt_cmd != "fzf" else ""
    try:
        p = PromptX(prompt_cmd=prompt_cmd, default_args=prompt_args)
        choice = p.ask(options=options, prompt=prompt)
    except (PromptXError, PromptXCmdError, PromptXSelectError) as err:
        print(err, sys.stderr)
        return None

    # Search for the chosen instance and return it's pid
    for key, val in pid_dict.items():
        if val[1] == choice:
            return key

    return None



"""
    The MIT License (MIT)

    © 2016 Allon Hadaya <self@allon.nyc>

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
"""
import re
import subprocess


class DmenuError(Exception):
    """The base class for dmenu errors."""

    pass


class DmenuCommandError(DmenuError):
    """The dmenu command failed."""

    def __init__(self, args, error):
        super(DmenuCommandError, self).__init__(
            "The provided dmenu command could not be used (%s): %s" % (args, error)
        )


class DmenuUsageError(DmenuError):
    """The dmenu command does not support your usage."""

    def __init__(self, args, usage):
        super(DmenuUsageError, self).__init__(
            "This version of dmenu does not support your usage (%s):\n\n%s"
            % (args, usage)
        )


def version(command="dmenu"):
    """The dmenu command's version message.

    Raises:
        DmenuCommandError

    Example:

        >>> import dmenu
        >>> dmenu.version()
        'dmenu-4.5, \xc2\xa9 2006-2012 dmenu engineers, see LICENSE for details'
    """

    args = [command, "-v"]

    try:
        # start the dmenu process
        proc = subprocess.Popen(
            args,
            universal_newlines=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
    except OSError as err:
        # something went wrong with starting the process
        raise DmenuCommandError(args, err)

    if proc.wait() == 0:
        # version information from stdout
        return proc.stdout.read().rstrip("\n")

    # error from dmenu
    raise DmenuCommandError(args, proc.stderr.read())


def show(
    items,
    command="dmenu",
    command_args=[],
    case_insensitive=None,
    lines=None,
    monitor=None,
    prompt=None,
):
    """Present a dmenu to the user.

    Args:
        items (Iterable[str]):
            defines the menu items being presented to the user. items should
            not contain the newline character.

        command (Optional[str]):
            defines the path to the dmenu executable. Defaults to 'dmenu'.

        command_args (Iterable[str]):
            generic way to add arguments to 'command', e.g. to use
            command="rofi" and command_args=["-dmenu"]

        case_insensitive (Optional[bool]):
            dmenu matches menu items case insensitively.

        lines (Optional[int]):
            dmenu lists items vertically, with the given number of lines.

        monitor (Optional[int]):
            dmenu is displayed on the monitor number supplied. Monitor numbers
            are starting from 0.

        prompt (Optional[str]):
            defines the prompt to be displayed to the left of the input field.

    Raises:
        DmenuCommandError
        DmenuUsageError

    Returns:
        The user's selected menu item, their own typed item, or None if they hit escape.

    Examples:

        >>> import dmenu

        >>> dmenu.show(['a', 'b', 'c'])
        'a'  # user selected a

        >>> dmenu.show(['a', 'b', 'c'], prompt='pick a letter')
        'b'  # user selected b

        >>> dmenu.show(['a', 'b', 'c'])
        None  # user hit escape

        >>> dmenu.show(['a', 'b', 'c'])
        'd'  # user typed their own selection, d

        >>> dmenu.show(['a', 'b', 'c'], command='not_a_valid_dmenu')
        Traceback (most recent call last):
          ...
        dmenu.dmenu.DmenuCommandError: The provided dmenu command could not be used (['not_a_valid_dmenu']): [Errno 2] No such file or directory: 'not_a_valid_dmenu'

        >>> dmenu.show(['a', 'b', 'c'], monitor=2)
        Traceback (most recent call last):
          ...
        dmenu.dmenu.DmenuUsageError: This version of dmenu does not support your usage (['dmenu', '-m', '2']):
        usage: dmenu [-b] [-f] [-i] [-l lines] [-p prompt] [-fn font]
                     [-nb color] [-nf color] [-sb color] [-sf color] [-v]

        Consider configuring show using partial application:

        >>> import functools
        >>> show = functools.partial(dmenu.show, bottom=True)
        >>> show(['we', 'show', 'up', 'below'])
        >>> show(['us', 'too'])
    """

    # construct args

    args = [command] + command_args

    if case_insensitive:
        args.append("-i")

    if lines is not None:
        args.extend(("-l", str(lines)))

    if monitor is not None:
        args.extend(("-m", str(monitor)))

    if prompt is not None:
        args.extend(("-p", prompt))

    try:
        # start the dmenu process
        proc = subprocess.Popen(
            args,
            universal_newlines=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
    except OSError as err:
        # something went wrong with starting the process
        raise DmenuCommandError(args, err)

    # write items over to dmenu
    with proc.stdin:
        for item in items:
            proc.stdin.write(item)
            proc.stdin.write("\n")

    if proc.wait() == 0:
        # user made a selection
        return proc.stdout.read().rstrip("\n")

    stderr = proc.stderr.read()

    if stderr == "":
        # user hit escape
        return None

    if re.match("usage", stderr, re.I):
        # usage error
        raise DmenuUsageError(args, stderr)

    # other error from dmenu
    raise DmenuCommandError(args, stderr)



#!/usr/bin/env python3
import re
import shlex
import subprocess
from typing import Optional, Iterable


class PromptXError(Exception):
    """
    Raised if OS error when trying to execute constructed prompt command
    or if prompt command returns non-zero value and stderr is not empty.
    """

    def __init__(self, cmd, err):
        self.cmd = cmd
        self.err = err
        self.message = (
            f"Failed to execute constructed command: {self.cmd}\nError: {self.err}"
        )
        super().__init__(self.message)


class PromptXCmdError(Exception):
    """
    Raised if PromptX is given a prompt command that is not fzf, dmenu, or rofi.
    """

    def __init__(self, prompt):
        self.prompt = prompt
        self.message = (
            f"PromptX does not yet handle the given prompt command: {self.prompt}"
        )
        super().__init__(self.message)


class PromptXSelectError(Exception):
    """
    Raised if select value is not "first", "last", or "all"
    """

    def __init__(self, select):
        self.select = select
        self.message = f"Invalid select option given: {self.select}"
        self.message = "\n".join(
            [self.message, 'Valid options are "first", "last", and "all"']
        )
        super().__init__(self.message)


class PromptX:
    """
    Prompt object that makes using dmenu, fzf, or rofi as easy as using
    it from the command line. To initialize the object give the desired
    prompt command you want to use along with any default args to pass.
    No need to pass a '--prompt=' or '-p' flag as this will be added if
    you give a prompt value when using the ask() method. Any other arg
    must be given explicitly in the form of a string. It will be properly
    split for you. Some example usage:

        >>> p = PromptX(prompt_cmd="dmenu", default_args="-l 20 -i")
        >>> choices = [ "1", "2", "3" ]
        >>> response = p.ask(choices)
        >>> print(response)
        '1'
        >>> response = p.ask(choices, prompt="Pick a number:", select="all")
        >>> print(response)
        ['1', '3']

    One nice thing about PromptX is you can configure it once and use it as
    many times as needed. If you need any additional arguments for a specific
    prompt the ask() method happily accepts them while not altering the PromptX
    object.

    Arguments for PromptX():
        prompt_cmd(str):
            The prompt command to use. PromptX can handle dmenu, fzf, and
            rofi. A note on rofi, it requires the '-dmenu' flag to read from
            stdin. PromptX will add this for you.

        default_args(Optional[str]): Default = ""
            This is a string of args like you would pass at the command line.
            PromptX will parse these for you using shlex. If you want to use
            an Iterable to add arguments use the add_args() method which takes
            an Iterable object.

    Arguments for ask() method:
        options(Iterable):
            A list of options to present the user with.

        prompt(Optional[str]):
            The prompt to use when querying the user.  The necassary flag
            will be added automatically to based on the prompt_cmd given on
            initilization.

        additional_args(Optional[str]):
            Add additional arguments to the prompt command.  Any additional
            arguments will be added before the prompt flag. These arguments
            must be a string. They will be appropriately split, just provide
            them as you would from the command line.

        select(Optional[str]):
            User's may select multiple answers in any of the three supported
            prompt_cmds. So you don't have to guess the return type of ask()
            you can indicate which user selection you want ask() to return:
            "first": Default
                Use "first" if you want the first option the user selected.
                Returns a string.
            "last":
                Use "last" if you want the last option the user selected.
                Returns a string.
            "all":
                Use "all" if you want all options the user selected.
                Returns a list of all selected options.

        deliminator(Optional[str]):
            Default is "\n". This is the deliminator to use when joining your
            list of options.

    Arguments for add_args() method:
        additional_args(Iterable):
            This method will add these args to the base_cmd of the PromptX
            object.
    """

    def __init__(
        self,
        prompt_cmd: str,
        default_args: Optional[str] = "",
    ):
        # Check that we can handle the given command
        valid_prompt_cmds = ["dmenu", "fzf", "rofi"]
        if not prompt_cmd in valid_prompt_cmds:
            raise PromptXCmdError(prompt=prompt_cmd)
        # As PromptX works through stdin rofi needs the -dmenu flag to work
        if prompt_cmd == "rofi":
            default_args = " ".join((default_args, "-dmenu"))
        self.prompt_cmd = prompt_cmd
        self.default_args = default_args
        # Create a basic list of command args
        self.base_cmd = shlex.split(" ".join((prompt_cmd, default_args)))

    def ask(
        self,
        options: Iterable,
        prompt: Optional[str] = None,
        additional_args: Optional[str] = None,
        select: Optional[str] = "first",
        deliminator: Optional[str] = "\n",
    ):
        """
        Ask the user to make a selection from the given options

        Arguments:
            options(Iterable):
                A list of options to present the user with.

            prompt(Optional[str]):
                The prompt to use when querying the user.  The necassary flag
                will be added automatically to based on the prompt_cmd given on
                initilization.

            additional_args(Optional[str]):
                Add additional arguments to the prompt command.  Any additional
                arguments will be added before the prompt flag. These arguments
                must be a string. They will be appropriately split, just provide
                them as you would from the command line.

            select(Optional[str]):
                User's may select multiple answers in any of the three supported
                prompt_cmds. So you don't have to guess the return type of ask()
                you can indicate which user selection you want ask() to return:
                "first": Default
                    Use "first" if you want the first option the user selected.
                    Returns a string.
                "last":
                    Use "last" if you want the last option the user selected.
                    Returns a string.
                "all":
                    Use "all" if you want all options the user selected.
                    Returns a list of all selected options.

            deliminator(Optional[str]):
                Default is "\n". This is the deliminator to use when joining your
                list of options.
        """
        # If check that select is properly set
        if select not in ["first", "last", "all"]:
            raise PromptXSelectError(select)
        # Initialize the cmd value
        cmd = self.base_cmd
        # fzf uses stderr to show prompt so we need to check for that
        stderr_file = None if self.prompt_cmd == "fzf" else subprocess.PIPE
        if additional_args is not None:
            cmd.extend(shlex.split(additional_args))
        if self.prompt_cmd == "fzf" and prompt is not None:
            cmd.append(f"--prompt={prompt}")
        elif prompt is not None:
            cmd.extend(["-p", prompt])

        # Start prompt_cmd with given args
        try:
            prompt = subprocess.Popen(
                cmd,
                universal_newlines=True,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=stderr_file,
            )
        # Failed to execute constructed command
        except OSError as err:
            raise PromptXError(cmd=cmd, err=err)

        # Create one long string similar to choice=$(printf '%s\n' "$@" | dmenu) in bash
        opts_str = deliminator.join(map(str, options))
        # Open stdin and populate choices
        with prompt.stdin:
            prompt.stdin.write(opts_str)

        # Wait for selection
        if prompt.wait() == 0:
            # Process user selection
            selection = prompt.stdout.read().rstrip().splitlines()
            # Return the requested selection
            if select == "first":
                return selection[0]
            elif select == "last":
                return selection[-1]
            else:
                return selection

        # fzf has no error to read, return None
        if stderr_file == None:
            return None
        # dmenu does print to stderr so we read it and return None
        # unless an actual error was encountered
        stderr = prompt.stderr.read()
        # If no err return None as user hit escape
        if stderr == "":
            return None
        # Otherwise some error occured
        raise PromptXError(cmd, stderr)

    def add_args(self, additional_args: Iterable):
        """
        Add additional args to the base_cmd value of the PromptX object. Does
        not overwrite what was given when when PromptX was initialized.
        """
        for arg in additional_args:
            self.base_cmd.extend(shlex.split(arg))



#!/usr/bin/env python3
import sys
import time
from pathlib import Path
from .config import UserSettings
from .options import get_opts
from .prompts import zathura_prompt
from .utils import (
    write_pid_file,
    get_next_or_prev_file,
    NoFileOrDirectory,
)
from .zathura import Zathura
from .zsocket import send_to_socket


__license__ = "GPL-v3.0"
__program__ = "znp"
# Create zathura object
z = Zathura()


def open_next_or_prev(
    file_path,
    next_file=False,
    prev_file=False,
    prompt_cmd=None,
):
    """
    Open file in zathura optionally open the next file in
    the directory of the given file
    """
    # Try to get the next or prev file in the cwd
    try:
        next_or_prev_file = get_next_or_prev_file(
            current_file_path=file_path,
            next_file=next_file,
            prev_file=prev_file,
        )
        # If next_or_prev_file is None then file_path
        # is the only valid file in the cwd. Exit gracefully
        if not next_or_prev_file:
            return 0
    except NoFileOrDirectory as err:
        print(err, sys.stderr)
        return 1
    # If user has ewmh try and use it to get the pid
    if z.has_ewmh:
        try:
            z.ewmh_get_pid()
            z.open(file_path=next_or_prev_file, pid=z.epid)
            return 0
        # Failed so we contiue and get the pid the hard way
        except TypeError:
            pass
    # Try searching for the given file manually
    # If found open it and exit
    if z.zathura_search(z.pids, search_file=file_path):
        z.open(file_path=next_or_prev_file, pid=z.search_pid)
        # z.remove_tmp_files()
        return 0
    # No prompt_cmd given, exit
    if not prompt_cmd:
        # z.remove_tmp_files()
        return 1
    # Prompt user for their desired instance
    choice = zathura_prompt(pid_dict=z.pid_dict, prompt_cmd=prompt_cmd)
    # No selection made, exit
    if not choice:
        # z.remove_tmp_files()
        return 1
    # Try opening the next file and exit
    z.open(file_path=next_or_prev_file, pid=choice)
    # z.remove_tmp_files()
    return 0


def open_given_file(
    file_path,
    prompt_cmd,
    prompt_args=None,
    pid=None,
):
    """
    Open the given file in zathura. Function may run in several ways:
        1. znp file.pdf
        2. znp -P 123456 file.pdf
        3. znp -P 123456 /path/to/file.pdf
        4. znp /path/to/file.pdf
    """
    # Resolve file_path if needed
    file_path = str(Path(file_path).resolve())
    # If given a pid then check if it exists an open the
    # file in the given instance
    if pid is not None and pid in z.pids:
        z.open(file_path=file_path, pid=pid)
        return 0
    # No pids open zathura ourself
    elif z.pid_count == 0:
        z.open(file_path=file_path)
        return 0
    # If only one pid then open file there
    elif z.pid_count == 1:
        z.open(file_path=file_path, pid=z.pids[0])
        return 0
    # Multiple zathura instances
    # Search and prompt user
    if z.zathura_search(z.pids):
        # TODO decide if znp should check for no choice
        # currently the znp will open the file in a new instance
        # if none is selected. Kind of nice but maybe not a good
        # idea.
        choice = zathura_prompt(
            z.pid_dict,
            prompt_cmd=prompt_cmd,
            prompt_args=prompt_args,
        )
        z.open(file_path=file_path, pid=choice)
        return 0
    # Search failed for some reason
    return 1


def main():
    """
    Main function for znp
    """
    args = get_opts()
    user_files = {
        "conf_file": f"{__program__}.conf",
    }
    default_settings = {
        "prompt_cmd": "fzf",
        "prompt_args": "",
    }
    read_files = ["conf_file"]
    user = UserSettings(
        program=__program__,
        cmd_line_args=args,
        user_files=user_files,
        default_settings=default_settings,
        read_files=read_files,
    )
    args = user.cmd_args
    # Connect client to server_address to send args.file and args.pid
    # to the calling znp server
    if args.client is not None:
        return send_to_socket(
            pid=args.pid,
            parent_pid=args.source,
            file_path=args.file,
            server_address=args.client,
        )
    # User is opening next or prev file
    if args.next or args.prev:
        return open_next_or_prev(
            file_path=args.file,
            next_file=args.next,
            prev_file=args.prev,
            prompt_cmd=user.prompt_cmd,
        )
    # User just wants a the pid of a specific instance
    if args.query:
        return z.query(search_file=args.file)
    # User wants to open the given file in an optionally given zathura instance
    return open_given_file(
        file_path=args.file,
        prompt_cmd=user.prompt_cmd,
        prompt_args=user.prompt_args,
        pid=args.pid,
    )


if __name__ == "__main__":
    # Store main exit status
    sys.exit(main())



#!/usr/bin/env python3
import os
import signal
import socket
import sys
import time
from threading import Thread


class SocketListener:
    """
    Run a socket listener in the background and listen for x connections
    """

    def __init__(self, server_address, expected_messages=1):
        self.server_address = server_address
        self.expected = expected_messages
        self.recieved = 0
        self.pids = []
        self.pid_dict = {}
        self.setup()
        thread = Thread(target=self.run, args=())
        thread.daemon = False
        thread.start()

    def run(self):
        # Create storage to be used in loop
        pids = []
        pid_dict = {}
        # Create a UDS socket
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        # Bind the socket to the address
        sock.bind(self.server_address)
        # Chmod the socket
        os.chmod(self.server_address, mode=0o600)
        # Listen for incoming connections
        sock.listen(self.expected)
        # Loop until recieved the expected number of connections
        while self.recieved < self.expected:
            # Clean up data for new connection
            path = None
            pid = None
            # Listen for connection
            connection, client_address = sock.accept()
            # Connection found, try receiving info
            try:
                while True:
                    # Get header, should be a pid padded with zeros
                    if not pid:
                        pid = connection.recv(7)
                    # Store temp_data
                    temp_path = connection.recv(1024)
                    if temp_path:
                        # Set data to temp_data if none stored yet
                        if not path:
                            path = temp_path
                        # Else add it to already stored data
                        else:
                            path = path + temp_path
                    # Connection complete store info
                    else:
                        # Decode message
                        pid = pid.decode(encoding="utf-8")
                        pid = str(int(pid))
                        path = path.decode(encoding="utf-8")
                        # self.pids.append(header)
                        pids.append(pid)
                        # self.pid_dict[header] = []
                        pid_dict[pid] = []
                        # self.pid_dict[header].extend([path, f"{pid}: {path}"])
                        pid_dict[pid].extend([path, f"{pid}: {path}"])
                        break
            finally:
                # Clean up the connection
                connection.close()
                self.recieved += 1
        # Finished receiving data, store it
        self.pids = pids
        self.pid_dict = pid_dict

    def setup(self):
        try:
            os.unlink(self.server_address)
        except OSError:
            if os.path.exists(self.server_address):
                raise


def send_to_socket(pid, parent_pid, file_path, server_address):
    """
    Send message to socket.
    The pid is used as the header.
    The file_path is used as the message.
    The server_address is the address to send the message to.
    """
    # Set pid to 0000000 if none given
    if not pid:
        pid = "0000000"
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    # Try connecting
    try:
        sock.connect(server_address)
    except socket.error as msg:
        print(f"Received error: {msg}\nKilling server.", sys.stderr)
        os.kill(parent_pid, signal.SIGKILL)
        return 1
    # Try sending data
    try:
        pid = bytes(str(pid).zfill(7), encoding="utf-8")
        file_path = bytes(str(file_path), encoding="utf-8")
        sock.sendall(pid)
        sock.sendall(file_path)
    finally:
        sock.close()

    return 0
